sqlglot.dialects.snowflake API documentation

Edit on GitHub

# sqlglot.dialects.snowflake

View Source

      1from __future__ import annotations
      2
      3import typing as t
      4
      5from sqlglot import exp, generator, parser, tokens, transforms
      6from sqlglot.dialects.dialect import (
      7    Dialect,
      8    date_trunc_to_time,
      9    datestrtodate_sql,
     10    format_time_lambda,
     11    inline_array_sql,
     12    max_or_greatest,
     13    min_or_least,
     14    rename_func,
     15    timestamptrunc_sql,
     16    timestrtotime_sql,
     17    ts_or_ds_to_date_sql,
     18    var_map_sql,
     19)
     20from sqlglot.expressions import Literal
     21from sqlglot.helper import seq_get
     22from sqlglot.parser import binary_range_parser
     23from sqlglot.tokens import TokenType
     24
     25
     26def _check_int(s: str) -> bool:
     27    if s[0] in ("-", "+"):
     28        return s[1:].isdigit()
     29    return s.isdigit()
     30
     31
     32# from https://docs.snowflake.com/en/sql-reference/functions/to_timestamp.html
     33def _snowflake_to_timestamp(args: t.List) -> t.Union[exp.StrToTime, exp.UnixToTime]:
     34    if len(args) == 2:
     35        first_arg, second_arg = args
     36        if second_arg.is_string:
     37            # case: <string_expr> [ , <format> ]
     38            return format_time_lambda(exp.StrToTime, "snowflake")(args)
     39
     40        # case: <numeric_expr> [ , <scale> ]
     41        if second_arg.name not in ["0", "3", "9"]:
     42            raise ValueError(
     43                f"Scale for snowflake numeric timestamp is {second_arg}, but should be 0, 3, or 9"
     44            )
     45
     46        if second_arg.name == "0":
     47            timescale = exp.UnixToTime.SECONDS
     48        elif second_arg.name == "3":
     49            timescale = exp.UnixToTime.MILLIS
     50        elif second_arg.name == "9":
     51            timescale = exp.UnixToTime.MICROS
     52
     53        return exp.UnixToTime(this=first_arg, scale=timescale)
     54
     55    from sqlglot.optimizer.simplify import simplify_literals
     56
     57    # The first argument might be an expression like 40 * 365 * 86400, so we try to
     58    # reduce it using `simplify_literals` first and then check if it's a Literal.
     59    first_arg = seq_get(args, 0)
     60    if not isinstance(simplify_literals(first_arg, root=True), Literal):
     61        # case: <variant_expr>
     62        return format_time_lambda(exp.StrToTime, "snowflake", default=True)(args)
     63
     64    if first_arg.is_string:
     65        if _check_int(first_arg.this):
     66            # case: <integer>
     67            return exp.UnixToTime.from_arg_list(args)
     68
     69        # case: <date_expr>
     70        return format_time_lambda(exp.StrToTime, "snowflake", default=True)(args)
     71
     72    # case: <numeric_expr>
     73    return exp.UnixToTime.from_arg_list(args)
     74
     75
     76def _parse_object_construct(args: t.List) -> t.Union[exp.StarMap, exp.Struct]:
     77    expression = parser.parse_var_map(args)
     78
     79    if isinstance(expression, exp.StarMap):
     80        return expression
     81
     82    return exp.Struct(
     83        expressions=[
     84            t.cast(exp.Condition, k).eq(v) for k, v in zip(expression.keys, expression.values)
     85        ]
     86    )
     87
     88
     89def _parse_datediff(args: t.List) -> exp.DateDiff:
     90    return exp.DateDiff(this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0))
     91
     92
     93def _unix_to_time_sql(self: generator.Generator, expression: exp.UnixToTime) -> str:
     94    scale = expression.args.get("scale")
     95    timestamp = self.sql(expression, "this")
     96    if scale in [None, exp.UnixToTime.SECONDS]:
     97        return f"TO_TIMESTAMP({timestamp})"
     98    if scale == exp.UnixToTime.MILLIS:
     99        return f"TO_TIMESTAMP({timestamp}, 3)"
    100    if scale == exp.UnixToTime.MICROS:
    101        return f"TO_TIMESTAMP({timestamp}, 9)"
    102
    103    raise ValueError("Improper scale for timestamp")
    104
    105
    106# https://docs.snowflake.com/en/sql-reference/functions/date_part.html
    107# https://docs.snowflake.com/en/sql-reference/functions-date-time.html#label-supported-date-time-parts
    108def _parse_date_part(self: parser.Parser) -> t.Optional[exp.Expression]:
    109    this = self._parse_var() or self._parse_type()
    110
    111    if not this:
    112        return None
    113
    114    self._match(TokenType.COMMA)
    115    expression = self._parse_bitwise()
    116
    117    name = this.name.upper()
    118    if name.startswith("EPOCH"):
    119        if name.startswith("EPOCH_MILLISECOND"):
    120            scale = 10**3
    121        elif name.startswith("EPOCH_MICROSECOND"):
    122            scale = 10**6
    123        elif name.startswith("EPOCH_NANOSECOND"):
    124            scale = 10**9
    125        else:
    126            scale = None
    127
    128        ts = self.expression(exp.Cast, this=expression, to=exp.DataType.build("TIMESTAMP"))
    129        to_unix: exp.Expression = self.expression(exp.TimeToUnix, this=ts)
    130
    131        if scale:
    132            to_unix = exp.Mul(this=to_unix, expression=exp.Literal.number(scale))
    133
    134        return to_unix
    135
    136    return self.expression(exp.Extract, this=this, expression=expression)
    137
    138
    139# https://docs.snowflake.com/en/sql-reference/functions/div0
    140def _div0_to_if(args: t.List) -> exp.Expression:
    141    cond = exp.EQ(this=seq_get(args, 1), expression=exp.Literal.number(0))
    142    true = exp.Literal.number(0)
    143    false = exp.Div(this=seq_get(args, 0), expression=seq_get(args, 1))
    144    return exp.If(this=cond, true=true, false=false)
    145
    146
    147# https://docs.snowflake.com/en/sql-reference/functions/zeroifnull
    148def _zeroifnull_to_if(args: t.List) -> exp.Expression:
    149    cond = exp.Is(this=seq_get(args, 0), expression=exp.Null())
    150    return exp.If(this=cond, true=exp.Literal.number(0), false=seq_get(args, 0))
    151
    152
    153# https://docs.snowflake.com/en/sql-reference/functions/zeroifnull
    154def _nullifzero_to_if(args: t.List) -> exp.Expression:
    155    cond = exp.EQ(this=seq_get(args, 0), expression=exp.Literal.number(0))
    156    return exp.If(this=cond, true=exp.Null(), false=seq_get(args, 0))
    157
    158
    159def _datatype_sql(self: generator.Generator, expression: exp.DataType) -> str:
    160    if expression.is_type("array"):
    161        return "ARRAY"
    162    elif expression.is_type("map"):
    163        return "OBJECT"
    164    return self.datatype_sql(expression)
    165
    166
    167def _parse_convert_timezone(args: t.List) -> exp.Expression:
    168    if len(args) == 3:
    169        return exp.Anonymous(this="CONVERT_TIMEZONE", expressions=args)
    170    return exp.AtTimeZone(this=seq_get(args, 1), zone=seq_get(args, 0))
    171
    172
    173class Snowflake(Dialect):
    174    # https://docs.snowflake.com/en/sql-reference/identifiers-syntax
    175    RESOLVES_IDENTIFIERS_AS_UPPERCASE = True
    176    NULL_ORDERING = "nulls_are_large"
    177    TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
    178
    179    TIME_MAPPING = {
    180        "YYYY": "%Y",
    181        "yyyy": "%Y",
    182        "YY": "%y",
    183        "yy": "%y",
    184        "MMMM": "%B",
    185        "mmmm": "%B",
    186        "MON": "%b",
    187        "mon": "%b",
    188        "MM": "%m",
    189        "mm": "%m",
    190        "DD": "%d",
    191        "dd": "%-d",
    192        "DY": "%a",
    193        "dy": "%w",
    194        "HH24": "%H",
    195        "hh24": "%H",
    196        "HH12": "%I",
    197        "hh12": "%I",
    198        "MI": "%M",
    199        "mi": "%M",
    200        "SS": "%S",
    201        "ss": "%S",
    202        "FF": "%f",
    203        "ff": "%f",
    204        "FF6": "%f",
    205        "ff6": "%f",
    206    }
    207
    208    class Parser(parser.Parser):
    209        IDENTIFY_PIVOT_STRINGS = True
    210
    211        FUNCTIONS = {
    212            **parser.Parser.FUNCTIONS,
    213            "ARRAYAGG": exp.ArrayAgg.from_arg_list,
    214            "ARRAY_CONSTRUCT": exp.Array.from_arg_list,
    215            "ARRAY_TO_STRING": exp.ArrayJoin.from_arg_list,
    216            "CONVERT_TIMEZONE": _parse_convert_timezone,
    217            "DATE_TRUNC": date_trunc_to_time,
    218            "DATEADD": lambda args: exp.DateAdd(
    219                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    220            ),
    221            "DATEDIFF": _parse_datediff,
    222            "DIV0": _div0_to_if,
    223            "IFF": exp.If.from_arg_list,
    224            "NULLIFZERO": _nullifzero_to_if,
    225            "OBJECT_CONSTRUCT": _parse_object_construct,
    226            "RLIKE": exp.RegexpLike.from_arg_list,
    227            "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
    228            "TIMEDIFF": _parse_datediff,
    229            "TIMESTAMPDIFF": _parse_datediff,
    230            "TO_ARRAY": exp.Array.from_arg_list,
    231            "TO_VARCHAR": exp.ToChar.from_arg_list,
    232            "TO_TIMESTAMP": _snowflake_to_timestamp,
    233            "ZEROIFNULL": _zeroifnull_to_if,
    234        }
    235
    236        FUNCTION_PARSERS = {
    237            **parser.Parser.FUNCTION_PARSERS,
    238            "DATE_PART": _parse_date_part,
    239        }
    240        FUNCTION_PARSERS.pop("TRIM")
    241
    242        FUNC_TOKENS = {
    243            *parser.Parser.FUNC_TOKENS,
    244            TokenType.RLIKE,
    245            TokenType.TABLE,
    246        }
    247
    248        COLUMN_OPERATORS = {
    249            **parser.Parser.COLUMN_OPERATORS,
    250            TokenType.COLON: lambda self, this, path: self.expression(
    251                exp.Bracket, this=this, expressions=[path]
    252            ),
    253        }
    254
    255        TIMESTAMPS = parser.Parser.TIMESTAMPS.copy() - {TokenType.TIME}
    256
    257        RANGE_PARSERS = {
    258            **parser.Parser.RANGE_PARSERS,
    259            TokenType.LIKE_ANY: binary_range_parser(exp.LikeAny),
    260            TokenType.ILIKE_ANY: binary_range_parser(exp.ILikeAny),
    261        }
    262
    263        ALTER_PARSERS = {
    264            **parser.Parser.ALTER_PARSERS,
    265            "SET": lambda self: self._parse_set(tag=self._match_text_seq("TAG")),
    266            "UNSET": lambda self: self.expression(
    267                exp.Set,
    268                tag=self._match_text_seq("TAG"),
    269                expressions=self._parse_csv(self._parse_id_var),
    270                unset=True,
    271            ),
    272        }
    273
    274        def _parse_id_var(
    275            self,
    276            any_token: bool = True,
    277            tokens: t.Optional[t.Collection[TokenType]] = None,
    278        ) -> t.Optional[exp.Expression]:
    279            if self._match_text_seq("IDENTIFIER", "("):
    280                identifier = (
    281                    super()._parse_id_var(any_token=any_token, tokens=tokens)
    282                    or self._parse_string()
    283                )
    284                self._match_r_paren()
    285                return self.expression(exp.Anonymous, this="IDENTIFIER", expressions=[identifier])
    286
    287            return super()._parse_id_var(any_token=any_token, tokens=tokens)
    288
    289    class Tokenizer(tokens.Tokenizer):
    290        QUOTES = ["'", "$$"]
    291        STRING_ESCAPES = ["\\", "'"]
    292        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    293        COMMENTS = ["--", "//", ("/*", "*/")]
    294
    295        KEYWORDS = {
    296            **tokens.Tokenizer.KEYWORDS,
    297            "BYTEINT": TokenType.INT,
    298            "CHAR VARYING": TokenType.VARCHAR,
    299            "CHARACTER VARYING": TokenType.VARCHAR,
    300            "EXCLUDE": TokenType.EXCEPT,
    301            "ILIKE ANY": TokenType.ILIKE_ANY,
    302            "LIKE ANY": TokenType.LIKE_ANY,
    303            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    304            "MINUS": TokenType.EXCEPT,
    305            "NCHAR VARYING": TokenType.VARCHAR,
    306            "PUT": TokenType.COMMAND,
    307            "RENAME": TokenType.REPLACE,
    308            "SAMPLE": TokenType.TABLE_SAMPLE,
    309            "TIMESTAMP_LTZ": TokenType.TIMESTAMPLTZ,
    310            "TIMESTAMP_NTZ": TokenType.TIMESTAMP,
    311            "TIMESTAMP_TZ": TokenType.TIMESTAMPTZ,
    312            "TIMESTAMPNTZ": TokenType.TIMESTAMP,
    313            "TOP": TokenType.TOP,
    314        }
    315
    316        SINGLE_TOKENS = {
    317            **tokens.Tokenizer.SINGLE_TOKENS,
    318            "$": TokenType.PARAMETER,
    319        }
    320
    321        VAR_SINGLE_TOKENS = {"$"}
    322
    323    class Generator(generator.Generator):
    324        PARAMETER_TOKEN = "$"
    325        MATCHED_BY_SOURCE = False
    326        SINGLE_STRING_INTERVAL = True
    327        JOIN_HINTS = False
    328        TABLE_HINTS = False
    329
    330        TRANSFORMS = {
    331            **generator.Generator.TRANSFORMS,
    332            exp.Array: inline_array_sql,
    333            exp.ArrayConcat: rename_func("ARRAY_CAT"),
    334            exp.ArrayJoin: rename_func("ARRAY_TO_STRING"),
    335            exp.AtTimeZone: lambda self, e: self.func(
    336                "CONVERT_TIMEZONE", e.args.get("zone"), e.this
    337            ),
    338            exp.DateAdd: lambda self, e: self.func("DATEADD", e.text("unit"), e.expression, e.this),
    339            exp.DateDiff: lambda self, e: self.func(
    340                "DATEDIFF", e.text("unit"), e.expression, e.this
    341            ),
    342            exp.DateStrToDate: datestrtodate_sql,
    343            exp.DataType: _datatype_sql,
    344            exp.DayOfWeek: rename_func("DAYOFWEEK"),
    345            exp.Extract: rename_func("DATE_PART"),
    346            exp.If: rename_func("IFF"),
    347            exp.LogicalAnd: rename_func("BOOLAND_AGG"),
    348            exp.LogicalOr: rename_func("BOOLOR_AGG"),
    349            exp.Map: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
    350            exp.Max: max_or_greatest,
    351            exp.Min: min_or_least,
    352            exp.PartitionedByProperty: lambda self, e: f"PARTITION BY {self.sql(e, 'this')}",
    353            exp.Select: transforms.preprocess([transforms.eliminate_distinct_on]),
    354            exp.StarMap: rename_func("OBJECT_CONSTRUCT"),
    355            exp.StrPosition: lambda self, e: self.func(
    356                "POSITION", e.args.get("substr"), e.this, e.args.get("position")
    357            ),
    358            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
    359            exp.Struct: lambda self, e: self.func(
    360                "OBJECT_CONSTRUCT",
    361                *(arg for expression in e.expressions for arg in expression.flatten()),
    362            ),
    363            exp.TimeStrToTime: timestrtotime_sql,
    364            exp.TimeToUnix: lambda self, e: f"EXTRACT(epoch_second FROM {self.sql(e, 'this')})",
    365            exp.TimeToStr: lambda self, e: self.func(
    366                "TO_CHAR", exp.cast(e.this, "timestamp"), self.format_time(e)
    367            ),
    368            exp.TimestampTrunc: timestamptrunc_sql,
    369            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    370            exp.Trim: lambda self, e: self.func("TRIM", e.this, e.expression),
    371            exp.TsOrDsToDate: ts_or_ds_to_date_sql("snowflake"),
    372            exp.UnixToTime: _unix_to_time_sql,
    373            exp.VarMap: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
    374        }
    375
    376        TYPE_MAPPING = {
    377            **generator.Generator.TYPE_MAPPING,
    378            exp.DataType.Type.TIMESTAMP: "TIMESTAMPNTZ",
    379        }
    380
    381        STAR_MAPPING = {
    382            "except": "EXCLUDE",
    383            "replace": "RENAME",
    384        }
    385
    386        PROPERTIES_LOCATION = {
    387            **generator.Generator.PROPERTIES_LOCATION,
    388            exp.SetProperty: exp.Properties.Location.UNSUPPORTED,
    389            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    390        }
    391
    392        def except_op(self, expression: exp.Except) -> str:
    393            if not expression.args.get("distinct", False):
    394                self.unsupported("EXCEPT with All is not supported in Snowflake")
    395            return super().except_op(expression)
    396
    397        def intersect_op(self, expression: exp.Intersect) -> str:
    398            if not expression.args.get("distinct", False):
    399                self.unsupported("INTERSECT with All is not supported in Snowflake")
    400            return super().intersect_op(expression)
    401
    402        def describe_sql(self, expression: exp.Describe) -> str:
    403            # Default to table if kind is unknown
    404            kind_value = expression.args.get("kind") or "TABLE"
    405            kind = f" {kind_value}" if kind_value else ""
    406            this = f" {self.sql(expression, 'this')}"
    407            return f"DESCRIBE{kind}{this}"
    408
    409        def generatedasidentitycolumnconstraint_sql(
    410            self, expression: exp.GeneratedAsIdentityColumnConstraint
    411        ) -> str:
    412            start = expression.args.get("start")
    413            start = f" START {start}" if start else ""
    414            increment = expression.args.get("increment")
    415            increment = f" INCREMENT {increment}" if increment else ""
    416            return f"AUTOINCREMENT{start}{increment}"

class Snowflake(sqlglot.dialects.dialect.Dialect): View Source

    174class Snowflake(Dialect):
    175    # https://docs.snowflake.com/en/sql-reference/identifiers-syntax
    176    RESOLVES_IDENTIFIERS_AS_UPPERCASE = True
    177    NULL_ORDERING = "nulls_are_large"
    178    TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
    179
    180    TIME_MAPPING = {
    181        "YYYY": "%Y",
    182        "yyyy": "%Y",
    183        "YY": "%y",
    184        "yy": "%y",
    185        "MMMM": "%B",
    186        "mmmm": "%B",
    187        "MON": "%b",
    188        "mon": "%b",
    189        "MM": "%m",
    190        "mm": "%m",
    191        "DD": "%d",
    192        "dd": "%-d",
    193        "DY": "%a",
    194        "dy": "%w",
    195        "HH24": "%H",
    196        "hh24": "%H",
    197        "HH12": "%I",
    198        "hh12": "%I",
    199        "MI": "%M",
    200        "mi": "%M",
    201        "SS": "%S",
    202        "ss": "%S",
    203        "FF": "%f",
    204        "ff": "%f",
    205        "FF6": "%f",
    206        "ff6": "%f",
    207    }
    208
    209    class Parser(parser.Parser):
    210        IDENTIFY_PIVOT_STRINGS = True
    211
    212        FUNCTIONS = {
    213            **parser.Parser.FUNCTIONS,
    214            "ARRAYAGG": exp.ArrayAgg.from_arg_list,
    215            "ARRAY_CONSTRUCT": exp.Array.from_arg_list,
    216            "ARRAY_TO_STRING": exp.ArrayJoin.from_arg_list,
    217            "CONVERT_TIMEZONE": _parse_convert_timezone,
    218            "DATE_TRUNC": date_trunc_to_time,
    219            "DATEADD": lambda args: exp.DateAdd(
    220                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    221            ),
    222            "DATEDIFF": _parse_datediff,
    223            "DIV0": _div0_to_if,
    224            "IFF": exp.If.from_arg_list,
    225            "NULLIFZERO": _nullifzero_to_if,
    226            "OBJECT_CONSTRUCT": _parse_object_construct,
    227            "RLIKE": exp.RegexpLike.from_arg_list,
    228            "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
    229            "TIMEDIFF": _parse_datediff,
    230            "TIMESTAMPDIFF": _parse_datediff,
    231            "TO_ARRAY": exp.Array.from_arg_list,
    232            "TO_VARCHAR": exp.ToChar.from_arg_list,
    233            "TO_TIMESTAMP": _snowflake_to_timestamp,
    234            "ZEROIFNULL": _zeroifnull_to_if,
    235        }
    236
    237        FUNCTION_PARSERS = {
    238            **parser.Parser.FUNCTION_PARSERS,
    239            "DATE_PART": _parse_date_part,
    240        }
    241        FUNCTION_PARSERS.pop("TRIM")
    242
    243        FUNC_TOKENS = {
    244            *parser.Parser.FUNC_TOKENS,
    245            TokenType.RLIKE,
    246            TokenType.TABLE,
    247        }
    248
    249        COLUMN_OPERATORS = {
    250            **parser.Parser.COLUMN_OPERATORS,
    251            TokenType.COLON: lambda self, this, path: self.expression(
    252                exp.Bracket, this=this, expressions=[path]
    253            ),
    254        }
    255
    256        TIMESTAMPS = parser.Parser.TIMESTAMPS.copy() - {TokenType.TIME}
    257
    258        RANGE_PARSERS = {
    259            **parser.Parser.RANGE_PARSERS,
    260            TokenType.LIKE_ANY: binary_range_parser(exp.LikeAny),
    261            TokenType.ILIKE_ANY: binary_range_parser(exp.ILikeAny),
    262        }
    263
    264        ALTER_PARSERS = {
    265            **parser.Parser.ALTER_PARSERS,
    266            "SET": lambda self: self._parse_set(tag=self._match_text_seq("TAG")),
    267            "UNSET": lambda self: self.expression(
    268                exp.Set,
    269                tag=self._match_text_seq("TAG"),
    270                expressions=self._parse_csv(self._parse_id_var),
    271                unset=True,
    272            ),
    273        }
    274
    275        def _parse_id_var(
    276            self,
    277            any_token: bool = True,
    278            tokens: t.Optional[t.Collection[TokenType]] = None,
    279        ) -> t.Optional[exp.Expression]:
    280            if self._match_text_seq("IDENTIFIER", "("):
    281                identifier = (
    282                    super()._parse_id_var(any_token=any_token, tokens=tokens)
    283                    or self._parse_string()
    284                )
    285                self._match_r_paren()
    286                return self.expression(exp.Anonymous, this="IDENTIFIER", expressions=[identifier])
    287
    288            return super()._parse_id_var(any_token=any_token, tokens=tokens)
    289
    290    class Tokenizer(tokens.Tokenizer):
    291        QUOTES = ["'", "$$"]
    292        STRING_ESCAPES = ["\\", "'"]
    293        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    294        COMMENTS = ["--", "//", ("/*", "*/")]
    295
    296        KEYWORDS = {
    297            **tokens.Tokenizer.KEYWORDS,
    298            "BYTEINT": TokenType.INT,
    299            "CHAR VARYING": TokenType.VARCHAR,
    300            "CHARACTER VARYING": TokenType.VARCHAR,
    301            "EXCLUDE": TokenType.EXCEPT,
    302            "ILIKE ANY": TokenType.ILIKE_ANY,
    303            "LIKE ANY": TokenType.LIKE_ANY,
    304            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    305            "MINUS": TokenType.EXCEPT,
    306            "NCHAR VARYING": TokenType.VARCHAR,
    307            "PUT": TokenType.COMMAND,
    308            "RENAME": TokenType.REPLACE,
    309            "SAMPLE": TokenType.TABLE_SAMPLE,
    310            "TIMESTAMP_LTZ": TokenType.TIMESTAMPLTZ,
    311            "TIMESTAMP_NTZ": TokenType.TIMESTAMP,
    312            "TIMESTAMP_TZ": TokenType.TIMESTAMPTZ,
    313            "TIMESTAMPNTZ": TokenType.TIMESTAMP,
    314            "TOP": TokenType.TOP,
    315        }
    316
    317        SINGLE_TOKENS = {
    318            **tokens.Tokenizer.SINGLE_TOKENS,
    319            "$": TokenType.PARAMETER,
    320        }
    321
    322        VAR_SINGLE_TOKENS = {"$"}
    323
    324    class Generator(generator.Generator):
    325        PARAMETER_TOKEN = "$"
    326        MATCHED_BY_SOURCE = False
    327        SINGLE_STRING_INTERVAL = True
    328        JOIN_HINTS = False
    329        TABLE_HINTS = False
    330
    331        TRANSFORMS = {
    332            **generator.Generator.TRANSFORMS,
    333            exp.Array: inline_array_sql,
    334            exp.ArrayConcat: rename_func("ARRAY_CAT"),
    335            exp.ArrayJoin: rename_func("ARRAY_TO_STRING"),
    336            exp.AtTimeZone: lambda self, e: self.func(
    337                "CONVERT_TIMEZONE", e.args.get("zone"), e.this
    338            ),
    339            exp.DateAdd: lambda self, e: self.func("DATEADD", e.text("unit"), e.expression, e.this),
    340            exp.DateDiff: lambda self, e: self.func(
    341                "DATEDIFF", e.text("unit"), e.expression, e.this
    342            ),
    343            exp.DateStrToDate: datestrtodate_sql,
    344            exp.DataType: _datatype_sql,
    345            exp.DayOfWeek: rename_func("DAYOFWEEK"),
    346            exp.Extract: rename_func("DATE_PART"),
    347            exp.If: rename_func("IFF"),
    348            exp.LogicalAnd: rename_func("BOOLAND_AGG"),
    349            exp.LogicalOr: rename_func("BOOLOR_AGG"),
    350            exp.Map: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
    351            exp.Max: max_or_greatest,
    352            exp.Min: min_or_least,
    353            exp.PartitionedByProperty: lambda self, e: f"PARTITION BY {self.sql(e, 'this')}",
    354            exp.Select: transforms.preprocess([transforms.eliminate_distinct_on]),
    355            exp.StarMap: rename_func("OBJECT_CONSTRUCT"),
    356            exp.StrPosition: lambda self, e: self.func(
    357                "POSITION", e.args.get("substr"), e.this, e.args.get("position")
    358            ),
    359            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
    360            exp.Struct: lambda self, e: self.func(
    361                "OBJECT_CONSTRUCT",
    362                *(arg for expression in e.expressions for arg in expression.flatten()),
    363            ),
    364            exp.TimeStrToTime: timestrtotime_sql,
    365            exp.TimeToUnix: lambda self, e: f"EXTRACT(epoch_second FROM {self.sql(e, 'this')})",
    366            exp.TimeToStr: lambda self, e: self.func(
    367                "TO_CHAR", exp.cast(e.this, "timestamp"), self.format_time(e)
    368            ),
    369            exp.TimestampTrunc: timestamptrunc_sql,
    370            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    371            exp.Trim: lambda self, e: self.func("TRIM", e.this, e.expression),
    372            exp.TsOrDsToDate: ts_or_ds_to_date_sql("snowflake"),
    373            exp.UnixToTime: _unix_to_time_sql,
    374            exp.VarMap: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
    375        }
    376
    377        TYPE_MAPPING = {
    378            **generator.Generator.TYPE_MAPPING,
    379            exp.DataType.Type.TIMESTAMP: "TIMESTAMPNTZ",
    380        }
    381
    382        STAR_MAPPING = {
    383            "except": "EXCLUDE",
    384            "replace": "RENAME",
    385        }
    386
    387        PROPERTIES_LOCATION = {
    388            **generator.Generator.PROPERTIES_LOCATION,
    389            exp.SetProperty: exp.Properties.Location.UNSUPPORTED,
    390            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    391        }
    392
    393        def except_op(self, expression: exp.Except) -> str:
    394            if not expression.args.get("distinct", False):
    395                self.unsupported("EXCEPT with All is not supported in Snowflake")
    396            return super().except_op(expression)
    397
    398        def intersect_op(self, expression: exp.Intersect) -> str:
    399            if not expression.args.get("distinct", False):
    400                self.unsupported("INTERSECT with All is not supported in Snowflake")
    401            return super().intersect_op(expression)
    402
    403        def describe_sql(self, expression: exp.Describe) -> str:
    404            # Default to table if kind is unknown
    405            kind_value = expression.args.get("kind") or "TABLE"
    406            kind = f" {kind_value}" if kind_value else ""
    407            this = f" {self.sql(expression, 'this')}"
    408            return f"DESCRIBE{kind}{this}"
    409
    410        def generatedasidentitycolumnconstraint_sql(
    411            self, expression: exp.GeneratedAsIdentityColumnConstraint
    412        ) -> str:
    413            start = expression.args.get("start")
    414            start = f" START {start}" if start else ""
    415            increment = expression.args.get("increment")
    416            increment = f" INCREMENT {increment}" if increment else ""
    417            return f"AUTOINCREMENT{start}{increment}"

RESOLVES_IDENTIFIERS_AS_UPPERCASE: Optional[bool] = True
NULL_ORDERING = 'nulls_are_large'
TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
TIME_MAPPING: Dict[str, str] = {'YYYY': '%Y', 'yyyy': '%Y', 'YY': '%y', 'yy': '%y', 'MMMM': '%B', 'mmmm': '%B', 'MON': '%b', 'mon': '%b', 'MM': '%m', 'mm': '%m', 'DD': '%d', 'dd': '%-d', 'DY': '%a', 'dy': '%w', 'HH24': '%H', 'hh24': '%H', 'HH12': '%I', 'hh12': '%I', 'MI': '%M', 'mi': '%M', 'SS': '%S', 'ss': '%S', 'FF': '%f', 'ff': '%f', 'FF6': '%f', 'ff6': '%f'}
tokenizer_class = <class 'sqlglot.dialects.snowflake.Snowflake.Tokenizer'>
parser_class = <class 'sqlglot.dialects.snowflake.Snowflake.Parser'>
generator_class = <class 'sqlglot.dialects.snowflake.Snowflake.Generator'>
TIME_TRIE: Dict = {'Y': {'Y': {'Y': {'Y': {0: True}}, 0: True}}, 'y': {'y': {'y': {'y': {0: True}}, 0: True}}, 'M': {'M': {'M': {'M': {0: True}}, 0: True}, 'O': {'N': {0: True}}, 'I': {0: True}}, 'm': {'m': {'m': {'m': {0: True}}, 0: True}, 'o': {'n': {0: True}}, 'i': {0: True}}, 'D': {'D': {0: True}, 'Y': {0: True}}, 'd': {'d': {0: True}, 'y': {0: True}}, 'H': {'H': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'h': {'h': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'S': {'S': {0: True}}, 's': {'s': {0: True}}, 'F': {'F': {0: True, '6': {0: True}}}, 'f': {'f': {0: True, '6': {0: True}}}}
FORMAT_TRIE: Dict = {'Y': {'Y': {'Y': {'Y': {0: True}}, 0: True}}, 'y': {'y': {'y': {'y': {0: True}}, 0: True}}, 'M': {'M': {'M': {'M': {0: True}}, 0: True}, 'O': {'N': {0: True}}, 'I': {0: True}}, 'm': {'m': {'m': {'m': {0: True}}, 0: True}, 'o': {'n': {0: True}}, 'i': {0: True}}, 'D': {'D': {0: True}, 'Y': {0: True}}, 'd': {'d': {0: True}, 'y': {0: True}}, 'H': {'H': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'h': {'h': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'S': {'S': {0: True}}, 's': {'s': {0: True}}, 'F': {'F': {0: True, '6': {0: True}}}, 'f': {'f': {0: True, '6': {0: True}}}}
INVERSE_TIME_MAPPING: Dict[str, str] = {'%Y': 'yyyy', '%y': 'yy', '%B': 'mmmm', '%b': 'mon', '%m': 'mm', '%d': 'DD', '%-d': 'dd', '%a': 'DY', '%w': 'dy', '%H': 'hh24', '%I': 'hh12', '%M': 'mi', '%S': 'ss', '%f': 'ff6'}
INVERSE_TIME_TRIE: Dict = {'%': {'Y': {0: True}, 'y': {0: True}, 'B': {0: True}, 'b': {0: True}, 'm': {0: True}, 'd': {0: True}, '-': {'d': {0: True}}, 'a': {0: True}, 'w': {0: True}, 'H': {0: True}, 'I': {0: True}, 'M': {0: True}, 'S': {0: True}, 'f': {0: True}}}
QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
BIT_START = None
BIT_END = None
HEX_START = "x'"
HEX_END = "'"
BYTE_START = None
BYTE_END = None
RAW_START = None
RAW_END = None

##### Inherited Members

sqlglot.dialects.dialect.Dialect
  INDEX_OFFSET
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  STRICT_STRING_CONCAT
  NORMALIZE_FUNCTIONS
  DATE_FORMAT
  DATEINT_FORMAT
  FORMAT_MAPPING
  get_or_raise
  format_time
  normalize_identifier
  case_sensitive
  can_identify
  quote_identifier
  parse
  parse_into
  generate
  transpile
  tokenize
  tokenizer
  parser
  generator

class Snowflake.Parser(sqlglot.parser.Parser): View Source

    209    class Parser(parser.Parser):
    210        IDENTIFY_PIVOT_STRINGS = True
    211
    212        FUNCTIONS = {
    213            **parser.Parser.FUNCTIONS,
    214            "ARRAYAGG": exp.ArrayAgg.from_arg_list,
    215            "ARRAY_CONSTRUCT": exp.Array.from_arg_list,
    216            "ARRAY_TO_STRING": exp.ArrayJoin.from_arg_list,
    217            "CONVERT_TIMEZONE": _parse_convert_timezone,
    218            "DATE_TRUNC": date_trunc_to_time,
    219            "DATEADD": lambda args: exp.DateAdd(
    220                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    221            ),
    222            "DATEDIFF": _parse_datediff,
    223            "DIV0": _div0_to_if,
    224            "IFF": exp.If.from_arg_list,
    225            "NULLIFZERO": _nullifzero_to_if,
    226            "OBJECT_CONSTRUCT": _parse_object_construct,
    227            "RLIKE": exp.RegexpLike.from_arg_list,
    228            "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
    229            "TIMEDIFF": _parse_datediff,
    230            "TIMESTAMPDIFF": _parse_datediff,
    231            "TO_ARRAY": exp.Array.from_arg_list,
    232            "TO_VARCHAR": exp.ToChar.from_arg_list,
    233            "TO_TIMESTAMP": _snowflake_to_timestamp,
    234            "ZEROIFNULL": _zeroifnull_to_if,
    235        }
    236
    237        FUNCTION_PARSERS = {
    238            **parser.Parser.FUNCTION_PARSERS,
    239            "DATE_PART": _parse_date_part,
    240        }
    241        FUNCTION_PARSERS.pop("TRIM")
    242
    243        FUNC_TOKENS = {
    244            *parser.Parser.FUNC_TOKENS,
    245            TokenType.RLIKE,
    246            TokenType.TABLE,
    247        }
    248
    249        COLUMN_OPERATORS = {
    250            **parser.Parser.COLUMN_OPERATORS,
    251            TokenType.COLON: lambda self, this, path: self.expression(
    252                exp.Bracket, this=this, expressions=[path]
    253            ),
    254        }
    255
    256        TIMESTAMPS = parser.Parser.TIMESTAMPS.copy() - {TokenType.TIME}
    257
    258        RANGE_PARSERS = {
    259            **parser.Parser.RANGE_PARSERS,
    260            TokenType.LIKE_ANY: binary_range_parser(exp.LikeAny),
    261            TokenType.ILIKE_ANY: binary_range_parser(exp.ILikeAny),
    262        }
    263
    264        ALTER_PARSERS = {
    265            **parser.Parser.ALTER_PARSERS,
    266            "SET": lambda self: self._parse_set(tag=self._match_text_seq("TAG")),
    267            "UNSET": lambda self: self.expression(
    268                exp.Set,
    269                tag=self._match_text_seq("TAG"),
    270                expressions=self._parse_csv(self._parse_id_var),
    271                unset=True,
    272            ),
    273        }
    274
    275        def _parse_id_var(
    276            self,
    277            any_token: bool = True,
    278            tokens: t.Optional[t.Collection[TokenType]] = None,
    279        ) -> t.Optional[exp.Expression]:
    280            if self._match_text_seq("IDENTIFIER", "("):
    281                identifier = (
    282                    super()._parse_id_var(any_token=any_token, tokens=tokens)
    283                    or self._parse_string()
    284                )
    285                self._match_r_paren()
    286                return self.expression(exp.Anonymous, this="IDENTIFIER", expressions=[identifier])
    287
    288            return super()._parse_id_var(any_token=any_token, tokens=tokens)

Parser consumes a list of tokens produced by the Tokenizer and produces a parsed syntax tree.

###### Arguments:

- error_level: The desired error level. Default: ErrorLevel.IMMEDIATE
- error_message_context: Determines the amount of context to capture from a query string when displaying the error message (in number of characters). Default: 100
- max_errors: Maximum number of error messages to include in a raised ParseError. This is only relevant if error_level is ErrorLevel.RAISE. Default: 3

IDENTIFY_PIVOT_STRINGS = True
FUNCTIONS = {'ABS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Abs'>>, 'ANY_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_COUNT_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxQuantile'>>, 'ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAll'>>, 'ARRAY_ANY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAny'>>, 'ARRAY_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_JOIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayJoin'>>, 'ARRAY_SIZE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_SORT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySort'>>, 'ARRAY_SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySum'>>, 'ARRAY_UNION_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUnionAgg'>>, 'AVG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Avg'>>, 'CASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Case'>>, 'CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cast'>>, 'CAST_TO_STR_TYPE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CastToStrType'>>, 'CEIL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CEILING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'COALESCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'IFNULL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'NVL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Concat'>>, 'CONCAT_WS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ConcatWs'>>, 'COUNT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Count'>>, 'COUNT_IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'CURRENT_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDate'>>, 'CURRENT_DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDatetime'>>, 'CURRENT_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTime'>>, 'CURRENT_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'CURRENT_USER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentUser'>>, 'DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Date'>>, 'DATE_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateAdd'>>, 'DATEDIFF': <function _parse_datediff>, 'DATE_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATE_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateStrToDate'>>, 'DATE_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateSub'>>, 'DATE_TO_DATE_STR': <function Parser.<lambda>>, 'DATE_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateToDi'>>, 'DATE_TRUNC': <function date_trunc_to_time>, 'DATETIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeAdd'>>, 'DATETIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeDiff'>>, 'DATETIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeSub'>>, 'DATETIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeTrunc'>>, 'DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Day'>>, 'DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAYOFMONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAY_OF_WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAYOFWEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAY_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DAYOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DECODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Decode'>>, 'DI_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DiToDate'>>, 'ENCODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Encode'>>, 'EXP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Exp'>>, 'EXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Extract'>>, 'FLOOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Floor'>>, 'FROM_BASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase'>>, 'FROM_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase64'>>, 'GENERATE_SERIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'GREATEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Greatest'>>, 'GROUP_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GroupConcat'>>, 'HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hex'>>, 'HLL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hll'>>, 'IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'INITCAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Initcap'>>, 'JSONB_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtract'>>, 'JSONB_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtractScalar'>>, 'JSON_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONExtract'>>, 'JSON_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONExtractScalar'>>, 'JSON_FORMAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONFormat'>>, 'J_S_O_N_OBJECT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObject'>>, 'LAST_DATE_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDateOfMonth'>>, 'LEAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Least'>>, 'LEFT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Left'>>, 'LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEVENSHTEIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Levenshtein'>>, 'LN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ln'>>, 'LOG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log'>>, 'LOG10': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log10'>>, 'LOG2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log2'>>, 'LOGICAL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOLAND_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'LOGICAL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOLOR_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'LOWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'LCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'MD5': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5'>>, 'MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Map'>>, 'MAP_FROM_ENTRIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MapFromEntries'>>, 'MATCH_AGAINST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MatchAgainst'>>, 'MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Max'>>, 'MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Min'>>, 'MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Month'>>, 'NEXT_VALUE_FOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NextValueFor'>>, 'NUMBER_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NumberToStr'>>, 'NVL2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nvl2'>>, 'OPEN_J_S_O_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.OpenJSON'>>, 'PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParameterizedAgg'>>, 'PERCENTILE_CONT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileCont'>>, 'PERCENTILE_DISC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileDisc'>>, 'POSEXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Posexplode'>>, 'POWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'POW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quantile'>>, 'RANGE_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RangeN'>>, 'READ_CSV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ReadCSV'>>, 'REDUCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Reduce'>>, 'REGEXP_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpExtract'>>, 'REGEXP_I_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpILike'>>, 'REGEXP_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'REGEXP_SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpSplit'>>, 'REPEAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Repeat'>>, 'RIGHT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Right'>>, 'ROUND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Round'>>, 'ROW_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RowNumber'>>, 'SHA': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA1': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA2'>>, 'SAFE_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeConcat'>>, 'SAFE_DIVIDE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeDivide'>>, 'SET_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SetAgg'>>, 'SORT_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SortArray'>>, 'SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Split'>>, 'SQRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sqrt'>>, 'STANDARD_HASH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StandardHash'>>, 'STAR_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StarMap'>>, 'STDDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDDEV_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevPop'>>, 'STDDEV_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevSamp'>>, 'STR_POSITION': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrPosition'>>, 'STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToDate'>>, 'STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToTime'>>, 'STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToUnix'>>, 'STRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'STRUCT_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StructExtract'>>, 'SUBSTRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Substring'>>, 'SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sum'>>, 'TIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeAdd'>>, 'TIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeDiff'>>, 'TIME_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToDate'>>, 'TIME_STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToTime'>>, 'TIME_STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToUnix'>>, 'TIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeSub'>>, 'TIME_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToStr'>>, 'TIME_TO_TIME_STR': <function Parser.<lambda>>, 'TIME_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeTrunc'>>, 'TIMESTAMP_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampAdd'>>, 'TIMESTAMP_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampSub'>>, 'TIMESTAMP_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampTrunc'>>, 'TO_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToBase64'>>, 'TO_CHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToChar'>>, 'TRIM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Trim'>>, 'TRY_CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TryCast'>>, 'TS_OR_DI_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDiToDi'>>, 'TS_OR_DS_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsAdd'>>, 'TS_OR_DS_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToDate'>>, 'TS_OR_DS_TO_DATE_STR': <function Parser.<lambda>>, 'UNHEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'UNIX_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToStr'>>, 'UNIX_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTime'>>, 'UNIX_TO_TIME_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTimeStr'>>, 'UPPER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'UCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'VAR_MAP': <function parse_var_map>, 'VARIANCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'VAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Week'>>, 'WEEK_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WEEKOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WHEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.When'>>, 'X_M_L_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.XMLTable'>>, 'YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Year'>>, 'GLOB': <function Parser.<lambda>>, 'LIKE': <function parse_like>, 'ARRAYAGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_CONSTRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_TO_STRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayJoin'>>, 'CONVERT_TIMEZONE': <function _parse_convert_timezone>, 'DATEADD': <function Snowflake.Parser.<lambda>>, 'DIV0': <function _div0_to_if>, 'IFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'NULLIFZERO': <function _nullifzero_to_if>, 'OBJECT_CONSTRUCT': <function _parse_object_construct>, 'RLIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'SQUARE': <function Snowflake.Parser.<lambda>>, 'TIMEDIFF': <function _parse_datediff>, 'TIMESTAMPDIFF': <function _parse_datediff>, 'TO_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'TO_VARCHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToChar'>>, 'TO_TIMESTAMP': <function _snowflake_to_timestamp>, 'ZEROIFNULL': <function _zeroifnull_to_if>}
FUNCTION_PARSERS = {'ANY_VALUE': <function Parser.<lambda>>, 'CAST': <function Parser.<lambda>>, 'CONCAT': <function Parser.<lambda>>, 'CONVERT': <function Parser.<lambda>>, 'DECODE': <function Parser.<lambda>>, 'EXTRACT': <function Parser.<lambda>>, 'JSON_OBJECT': <function Parser.<lambda>>, 'LOG': <function Parser.<lambda>>, 'MATCH': <function Parser.<lambda>>, 'OPENJSON': <function Parser.<lambda>>, 'POSITION': <function Parser.<lambda>>, 'SAFE_CAST': <function Parser.<lambda>>, 'STRING_AGG': <function Parser.<lambda>>, 'SUBSTRING': <function Parser.<lambda>>, 'TRY_CAST': <function Parser.<lambda>>, 'TRY_CONVERT': <function Parser.<lambda>>, 'DATE_PART': <function _parse_date_part>}
FUNC_TOKENS = {<TokenType.UNNEST: 'UNNEST'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.INT: 'INT'>, <TokenType.UUID: 'UUID'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.SUPER: 'SUPER'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.OFFSET: 'OFFSET'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.JSON: 'JSON'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.MERGE: 'MERGE'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, <TokenType.RLIKE: 'RLIKE'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.SOME: 'SOME'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.CHAR: 'CHAR'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.IDENTIFIER: 'IDENTIFIER'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.JSONB: 'JSONB'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.BIT: 'BIT'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.TABLE: 'TABLE'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.TIME: 'TIME'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.ROW: 'ROW'>, <TokenType.MAP: 'MAP'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.RIGHT: 'RIGHT'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.WINDOW: 'WINDOW'>, <TokenType.VAR: 'VAR'>, <TokenType.MONEY: 'MONEY'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.UINT128: 'UINT128'>, <TokenType.ANY: 'ANY'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.ALL: 'ALL'>, <TokenType.FILTER: 'FILTER'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.INDEX: 'INDEX'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.DATE: 'DATE'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.FIRST: 'FIRST'>, <TokenType.XML: 'XML'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.INET: 'INET'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.GLOB: 'GLOB'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.RANGE: 'RANGE'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.UINT: 'UINT'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.ENUM: 'ENUM'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.ILIKE: 'ILIKE'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.TEXT: 'TEXT'>, <TokenType.INT128: 'INT128'>, <TokenType.UINT256: 'UINT256'>, <TokenType.LEFT: 'LEFT'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.BINARY: 'BINARY'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.LIKE: 'LIKE'>, <TokenType.INT256: 'INT256'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>}
COLUMN_OPERATORS = {<TokenType.DOT: 'DOT'>: None, <TokenType.DCOLON: 'DCOLON'>: <function Parser.<lambda>>, <TokenType.ARROW: 'ARROW'>: <function Parser.<lambda>>, <TokenType.DARROW: 'DARROW'>: <function Parser.<lambda>>, <TokenType.HASH_ARROW: 'HASH_ARROW'>: <function Parser.<lambda>>, <TokenType.DHASH_ARROW: 'DHASH_ARROW'>: <function Parser.<lambda>>, <TokenType.PLACEHOLDER: 'PLACEHOLDER'>: <function Parser.<lambda>>, <TokenType.COLON: 'COLON'>: <function Snowflake.Parser.<lambda>>}
TIMESTAMPS = {<TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>}
RANGE_PARSERS = {<TokenType.BETWEEN: 'BETWEEN'>: <function Parser.<lambda>>, <TokenType.GLOB: 'GLOB'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.ILIKE: 'ILIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.IN: 'IN'>: <function Parser.<lambda>>, <TokenType.IRLIKE: 'IRLIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.IS: 'IS'>: <function Parser.<lambda>>, <TokenType.LIKE: 'LIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.OVERLAPS: 'OVERLAPS'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.RLIKE: 'RLIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.SIMILAR_TO: 'SIMILAR_TO'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.LIKE_ANY: 'LIKE_ANY'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.ILIKE_ANY: 'ILIKE_ANY'>: <function binary_range_parser.<locals>.<lambda>>}
ALTER_PARSERS = {'ADD': <function Parser.<lambda>>, 'ALTER': <function Parser.<lambda>>, 'DELETE': <function Parser.<lambda>>, 'DROP': <function Parser.<lambda>>, 'RENAME': <function Parser.<lambda>>, 'SET': <function Snowflake.Parser.<lambda>>, 'UNSET': <function Snowflake.Parser.<lambda>>}
NULL_ORDERING: str = 'nulls_are_large'
SHOW_TRIE: Dict = {}
SET_TRIE: Dict = {'GLOBAL': {0: True}, 'LOCAL': {0: True}, 'SESSION': {0: True}, 'TRANSACTION': {0: True}}
FORMAT_TRIE: Dict = {'Y': {'Y': {'Y': {'Y': {0: True}}, 0: True}}, 'y': {'y': {'y': {'y': {0: True}}, 0: True}}, 'M': {'M': {'M': {'M': {0: True}}, 0: True}, 'O': {'N': {0: True}}, 'I': {0: True}}, 'm': {'m': {'m': {'m': {0: True}}, 0: True}, 'o': {'n': {0: True}}, 'i': {0: True}}, 'D': {'D': {0: True}, 'Y': {0: True}}, 'd': {'d': {0: True}, 'y': {0: True}}, 'H': {'H': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'h': {'h': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'S': {'S': {0: True}}, 's': {'s': {0: True}}, 'F': {'F': {0: True, '6': {0: True}}}, 'f': {'f': {0: True, '6': {0: True}}}}
TIME_MAPPING: Dict[str, str] = {'YYYY': '%Y', 'yyyy': '%Y', 'YY': '%y', 'yy': '%y', 'MMMM': '%B', 'mmmm': '%B', 'MON': '%b', 'mon': '%b', 'MM': '%m', 'mm': '%m', 'DD': '%d', 'dd': '%-d', 'DY': '%a', 'dy': '%w', 'HH24': '%H', 'hh24': '%H', 'HH12': '%I', 'hh12': '%I', 'MI': '%M', 'mi': '%M', 'SS': '%S', 'ss': '%S', 'FF': '%f', 'ff': '%f', 'FF6': '%f', 'ff6': '%f'}
TIME_TRIE: Dict = {'Y': {'Y': {'Y': {'Y': {0: True}}, 0: True}}, 'y': {'y': {'y': {'y': {0: True}}, 0: True}}, 'M': {'M': {'M': {'M': {0: True}}, 0: True}, 'O': {'N': {0: True}}, 'I': {0: True}}, 'm': {'m': {'m': {'m': {0: True}}, 0: True}, 'o': {'n': {0: True}}, 'i': {0: True}}, 'D': {'D': {0: True}, 'Y': {0: True}}, 'd': {'d': {0: True}, 'y': {0: True}}, 'H': {'H': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'h': {'h': {'2': {'4': {0: True}}, '1': {'2': {0: True}}}}, 'S': {'S': {0: True}}, 's': {'s': {0: True}}, 'F': {'F': {0: True, '6': {0: True}}}, 'f': {'f': {0: True, '6': {0: True}}}}

##### Inherited Members

sqlglot.parser.Parser
  Parser
  NO_PAREN_FUNCTIONS
  NESTED_TYPE_TOKENS
  ENUM_TYPE_TOKENS
  TYPE_TOKENS
  SUBQUERY_PREDICATES
  RESERVED_KEYWORDS
  DB_CREATABLES
  CREATABLES
  ID_VAR_TOKENS
  INTERVAL_VARS
  TABLE_ALIAS_TOKENS
  COMMENT_TABLE_ALIAS_TOKENS
  UPDATE_ALIAS_TOKENS
  TRIM_TYPES
  CONJUNCTION
  EQUALITY
  COMPARISON
  BITWISE
  TERM
  FACTOR
  SET_OPERATIONS
  JOIN_METHODS
  JOIN_SIDES
  JOIN_KINDS
  JOIN_HINTS
  LAMBDAS
  EXPRESSION_PARSERS
  STATEMENT_PARSERS
  UNARY_PARSERS
  PRIMARY_PARSERS
  PLACEHOLDER_PARSERS
  PROPERTY_PARSERS
  CONSTRAINT_PARSERS
  SCHEMA_UNNAMED_CONSTRAINTS
  NO_PAREN_FUNCTION_PARSERS
  FUNCTIONS_WITH_ALIASED_ARGS
  QUERY_MODIFIER_PARSERS
  SET_PARSERS
  SHOW_PARSERS
  TYPE_LITERAL_PARSERS
  MODIFIABLES
  DDL_SELECT_TOKENS
  PRE_VOLATILE_TOKENS
  TRANSACTION_KIND
  TRANSACTION_CHARACTERISTICS
  INSERT_ALTERNATIVES
  CLONE_KINDS
  TABLE_INDEX_HINT_TOKENS
  WINDOW_ALIAS_TOKENS
  WINDOW_BEFORE_PAREN_TOKENS
  WINDOW_SIDES
  ADD_CONSTRAINT_TOKENS
  STRICT_CAST
  CONCAT_NULL_OUTPUTS_STRING
  PREFIXED_PIVOT_COLUMNS
  LOG_BASE_FIRST
  LOG_DEFAULTS_TO_LN
  INDEX_OFFSET
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  STRICT_STRING_CONCAT
  FORMAT_MAPPING
  error_level
  error_message_context
  max_errors
  reset
  parse
  parse_into
  check_errors
  raise_error
  expression
  validate_expression
  errors
  sql

class Snowflake.Tokenizer(sqlglot.tokens.Tokenizer): View Source

    290    class Tokenizer(tokens.Tokenizer):
    291        QUOTES = ["'", "$$"]
    292        STRING_ESCAPES = ["\\", "'"]
    293        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    294        COMMENTS = ["--", "//", ("/*", "*/")]
    295
    296        KEYWORDS = {
    297            **tokens.Tokenizer.KEYWORDS,
    298            "BYTEINT": TokenType.INT,
    299            "CHAR VARYING": TokenType.VARCHAR,
    300            "CHARACTER VARYING": TokenType.VARCHAR,
    301            "EXCLUDE": TokenType.EXCEPT,
    302            "ILIKE ANY": TokenType.ILIKE_ANY,
    303            "LIKE ANY": TokenType.LIKE_ANY,
    304            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    305            "MINUS": TokenType.EXCEPT,
    306            "NCHAR VARYING": TokenType.VARCHAR,
    307            "PUT": TokenType.COMMAND,
    308            "RENAME": TokenType.REPLACE,
    309            "SAMPLE": TokenType.TABLE_SAMPLE,
    310            "TIMESTAMP_LTZ": TokenType.TIMESTAMPLTZ,
    311            "TIMESTAMP_NTZ": TokenType.TIMESTAMP,
    312            "TIMESTAMP_TZ": TokenType.TIMESTAMPTZ,
    313            "TIMESTAMPNTZ": TokenType.TIMESTAMP,
    314            "TOP": TokenType.TOP,
    315        }
    316
    317        SINGLE_TOKENS = {
    318            **tokens.Tokenizer.SINGLE_TOKENS,
    319            "$": TokenType.PARAMETER,
    320        }
    321
    322        VAR_SINGLE_TOKENS = {"$"}

QUOTES = ["'", '$$']
STRING_ESCAPES = ['\\', "'"]
HEX_STRINGS = [("x'", "'"), ("X'", "'")]
COMMENTS = ['--', '//', ('/*', '*/')]
KEYWORDS = {'{%': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%-': <TokenType.BLOCK_START: 'BLOCK_START'>, '%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '+%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '{{+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{{-': <TokenType.BLOCK_START: 'BLOCK_START'>, '+}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '/*+': <TokenType.HINT: 'HINT'>, '==': <TokenType.EQ: 'EQ'>, '::': <TokenType.DCOLON: 'DCOLON'>, '||': <TokenType.DPIPE: 'DPIPE'>, '>=': <TokenType.GTE: 'GTE'>, '<=': <TokenType.LTE: 'LTE'>, '<>': <TokenType.NEQ: 'NEQ'>, '!=': <TokenType.NEQ: 'NEQ'>, '<=>': <TokenType.NULLSAFE_EQ: 'NULLSAFE_EQ'>, '->': <TokenType.ARROW: 'ARROW'>, '->>': <TokenType.DARROW: 'DARROW'>, '=>': <TokenType.FARROW: 'FARROW'>, '#>': <TokenType.HASH_ARROW: 'HASH_ARROW'>, '#>>': <TokenType.DHASH_ARROW: 'DHASH_ARROW'>, '<->': <TokenType.LR_ARROW: 'LR_ARROW'>, '&&': <TokenType.DAMP: 'DAMP'>, 'ALL': <TokenType.ALL: 'ALL'>, 'ALWAYS': <TokenType.ALWAYS: 'ALWAYS'>, 'AND': <TokenType.AND: 'AND'>, 'ANTI': <TokenType.ANTI: 'ANTI'>, 'ANY': <TokenType.ANY: 'ANY'>, 'ASC': <TokenType.ASC: 'ASC'>, 'AS': <TokenType.ALIAS: 'ALIAS'>, 'ASOF': <TokenType.ASOF: 'ASOF'>, 'AUTOINCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'AUTO_INCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'BEGIN': <TokenType.BEGIN: 'BEGIN'>, 'BETWEEN': <TokenType.BETWEEN: 'BETWEEN'>, 'CACHE': <TokenType.CACHE: 'CACHE'>, 'UNCACHE': <TokenType.UNCACHE: 'UNCACHE'>, 'CASE': <TokenType.CASE: 'CASE'>, 'CHARACTER SET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'CLUSTER BY': <TokenType.CLUSTER_BY: 'CLUSTER_BY'>, 'COLLATE': <TokenType.COLLATE: 'COLLATE'>, 'COLUMN': <TokenType.COLUMN: 'COLUMN'>, 'COMMIT': <TokenType.COMMIT: 'COMMIT'>, 'CONSTRAINT': <TokenType.CONSTRAINT: 'CONSTRAINT'>, 'CREATE': <TokenType.CREATE: 'CREATE'>, 'CROSS': <TokenType.CROSS: 'CROSS'>, 'CUBE': <TokenType.CUBE: 'CUBE'>, 'CURRENT_DATE': <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, 'CURRENT_TIME': <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, 'CURRENT_TIMESTAMP': <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, 'CURRENT_USER': <TokenType.CURRENT_USER: 'CURRENT_USER'>, 'DATABASE': <TokenType.DATABASE: 'DATABASE'>, 'DEFAULT': <TokenType.DEFAULT: 'DEFAULT'>, 'DELETE': <TokenType.DELETE: 'DELETE'>, 'DESC': <TokenType.DESC: 'DESC'>, 'DESCRIBE': <TokenType.DESCRIBE: 'DESCRIBE'>, 'DISTINCT': <TokenType.DISTINCT: 'DISTINCT'>, 'DISTRIBUTE BY': <TokenType.DISTRIBUTE_BY: 'DISTRIBUTE_BY'>, 'DIV': <TokenType.DIV: 'DIV'>, 'DROP': <TokenType.DROP: 'DROP'>, 'ELSE': <TokenType.ELSE: 'ELSE'>, 'END': <TokenType.END: 'END'>, 'ESCAPE': <TokenType.ESCAPE: 'ESCAPE'>, 'EXCEPT': <TokenType.EXCEPT: 'EXCEPT'>, 'EXECUTE': <TokenType.EXECUTE: 'EXECUTE'>, 'EXISTS': <TokenType.EXISTS: 'EXISTS'>, 'FALSE': <TokenType.FALSE: 'FALSE'>, 'FETCH': <TokenType.FETCH: 'FETCH'>, 'FILTER': <TokenType.FILTER: 'FILTER'>, 'FIRST': <TokenType.FIRST: 'FIRST'>, 'FULL': <TokenType.FULL: 'FULL'>, 'FUNCTION': <TokenType.FUNCTION: 'FUNCTION'>, 'FOR': <TokenType.FOR: 'FOR'>, 'FOREIGN KEY': <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, 'FORMAT': <TokenType.FORMAT: 'FORMAT'>, 'FROM': <TokenType.FROM: 'FROM'>, 'GEOGRAPHY': <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, 'GEOMETRY': <TokenType.GEOMETRY: 'GEOMETRY'>, 'GLOB': <TokenType.GLOB: 'GLOB'>, 'GROUP BY': <TokenType.GROUP_BY: 'GROUP_BY'>, 'GROUPING SETS': <TokenType.GROUPING_SETS: 'GROUPING_SETS'>, 'HAVING': <TokenType.HAVING: 'HAVING'>, 'IF': <TokenType.IF: 'IF'>, 'ILIKE': <TokenType.ILIKE: 'ILIKE'>, 'IN': <TokenType.IN: 'IN'>, 'INDEX': <TokenType.INDEX: 'INDEX'>, 'INET': <TokenType.INET: 'INET'>, 'INNER': <TokenType.INNER: 'INNER'>, 'INSERT': <TokenType.INSERT: 'INSERT'>, 'INTERVAL': <TokenType.INTERVAL: 'INTERVAL'>, 'INTERSECT': <TokenType.INTERSECT: 'INTERSECT'>, 'INTO': <TokenType.INTO: 'INTO'>, 'IS': <TokenType.IS: 'IS'>, 'ISNULL': <TokenType.ISNULL: 'ISNULL'>, 'JOIN': <TokenType.JOIN: 'JOIN'>, 'KEEP': <TokenType.KEEP: 'KEEP'>, 'LATERAL': <TokenType.LATERAL: 'LATERAL'>, 'LEFT': <TokenType.LEFT: 'LEFT'>, 'LIKE': <TokenType.LIKE: 'LIKE'>, 'LIMIT': <TokenType.LIMIT: 'LIMIT'>, 'LOAD': <TokenType.LOAD: 'LOAD'>, 'LOCK': <TokenType.LOCK: 'LOCK'>, 'MERGE': <TokenType.MERGE: 'MERGE'>, 'NATURAL': <TokenType.NATURAL: 'NATURAL'>, 'NEXT': <TokenType.NEXT: 'NEXT'>, 'NEXT VALUE FOR': <TokenType.NEXT_VALUE_FOR: 'NEXT_VALUE_FOR'>, 'NOT': <TokenType.NOT: 'NOT'>, 'NOTNULL': <TokenType.NOTNULL: 'NOTNULL'>, 'NULL': <TokenType.NULL: 'NULL'>, 'OBJECT': <TokenType.OBJECT: 'OBJECT'>, 'OFFSET': <TokenType.OFFSET: 'OFFSET'>, 'ON': <TokenType.ON: 'ON'>, 'OR': <TokenType.OR: 'OR'>, 'ORDER BY': <TokenType.ORDER_BY: 'ORDER_BY'>, 'ORDINALITY': <TokenType.ORDINALITY: 'ORDINALITY'>, 'OUTER': <TokenType.OUTER: 'OUTER'>, 'OVER': <TokenType.OVER: 'OVER'>, 'OVERLAPS': <TokenType.OVERLAPS: 'OVERLAPS'>, 'OVERWRITE': <TokenType.OVERWRITE: 'OVERWRITE'>, 'PARTITION': <TokenType.PARTITION: 'PARTITION'>, 'PARTITION BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED_BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PERCENT': <TokenType.PERCENT: 'PERCENT'>, 'PIVOT': <TokenType.PIVOT: 'PIVOT'>, 'PRAGMA': <TokenType.PRAGMA: 'PRAGMA'>, 'PRIMARY KEY': <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, 'PROCEDURE': <TokenType.PROCEDURE: 'PROCEDURE'>, 'QUALIFY': <TokenType.QUALIFY: 'QUALIFY'>, 'RANGE': <TokenType.RANGE: 'RANGE'>, 'RECURSIVE': <TokenType.RECURSIVE: 'RECURSIVE'>, 'REGEXP': <TokenType.RLIKE: 'RLIKE'>, 'REPLACE': <TokenType.REPLACE: 'REPLACE'>, 'RETURNING': <TokenType.RETURNING: 'RETURNING'>, 'REFERENCES': <TokenType.REFERENCES: 'REFERENCES'>, 'RIGHT': <TokenType.RIGHT: 'RIGHT'>, 'RLIKE': <TokenType.RLIKE: 'RLIKE'>, 'ROLLBACK': <TokenType.ROLLBACK: 'ROLLBACK'>, 'ROLLUP': <TokenType.ROLLUP: 'ROLLUP'>, 'ROW': <TokenType.ROW: 'ROW'>, 'ROWS': <TokenType.ROWS: 'ROWS'>, 'SCHEMA': <TokenType.SCHEMA: 'SCHEMA'>, 'SELECT': <TokenType.SELECT: 'SELECT'>, 'SEMI': <TokenType.SEMI: 'SEMI'>, 'SET': <TokenType.SET: 'SET'>, 'SETTINGS': <TokenType.SETTINGS: 'SETTINGS'>, 'SHOW': <TokenType.SHOW: 'SHOW'>, 'SIMILAR TO': <TokenType.SIMILAR_TO: 'SIMILAR_TO'>, 'SOME': <TokenType.SOME: 'SOME'>, 'SORT BY': <TokenType.SORT_BY: 'SORT_BY'>, 'TABLE': <TokenType.TABLE: 'TABLE'>, 'TABLESAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TEMP': <TokenType.TEMPORARY: 'TEMPORARY'>, 'TEMPORARY': <TokenType.TEMPORARY: 'TEMPORARY'>, 'THEN': <TokenType.THEN: 'THEN'>, 'TRUE': <TokenType.TRUE: 'TRUE'>, 'UNION': <TokenType.UNION: 'UNION'>, 'UNNEST': <TokenType.UNNEST: 'UNNEST'>, 'UNPIVOT': <TokenType.UNPIVOT: 'UNPIVOT'>, 'UPDATE': <TokenType.UPDATE: 'UPDATE'>, 'USE': <TokenType.USE: 'USE'>, 'USING': <TokenType.USING: 'USING'>, 'UUID': <TokenType.UUID: 'UUID'>, 'VALUES': <TokenType.VALUES: 'VALUES'>, 'VIEW': <TokenType.VIEW: 'VIEW'>, 'VOLATILE': <TokenType.VOLATILE: 'VOLATILE'>, 'WHEN': <TokenType.WHEN: 'WHEN'>, 'WHERE': <TokenType.WHERE: 'WHERE'>, 'WINDOW': <TokenType.WINDOW: 'WINDOW'>, 'WITH': <TokenType.WITH: 'WITH'>, 'APPLY': <TokenType.APPLY: 'APPLY'>, 'ARRAY': <TokenType.ARRAY: 'ARRAY'>, 'BIT': <TokenType.BIT: 'BIT'>, 'BOOL': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BOOLEAN': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BYTE': <TokenType.TINYINT: 'TINYINT'>, 'TINYINT': <TokenType.TINYINT: 'TINYINT'>, 'SHORT': <TokenType.SMALLINT: 'SMALLINT'>, 'SMALLINT': <TokenType.SMALLINT: 'SMALLINT'>, 'INT2': <TokenType.SMALLINT: 'SMALLINT'>, 'INTEGER': <TokenType.INT: 'INT'>, 'INT': <TokenType.INT: 'INT'>, 'INT4': <TokenType.INT: 'INT'>, 'LONG': <TokenType.BIGINT: 'BIGINT'>, 'BIGINT': <TokenType.BIGINT: 'BIGINT'>, 'INT8': <TokenType.BIGINT: 'BIGINT'>, 'DEC': <TokenType.DECIMAL: 'DECIMAL'>, 'DECIMAL': <TokenType.DECIMAL: 'DECIMAL'>, 'BIGDECIMAL': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'BIGNUMERIC': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'MAP': <TokenType.MAP: 'MAP'>, 'NULLABLE': <TokenType.NULLABLE: 'NULLABLE'>, 'NUMBER': <TokenType.DECIMAL: 'DECIMAL'>, 'NUMERIC': <TokenType.DECIMAL: 'DECIMAL'>, 'FIXED': <TokenType.DECIMAL: 'DECIMAL'>, 'REAL': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT4': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT8': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE PRECISION': <TokenType.DOUBLE: 'DOUBLE'>, 'JSON': <TokenType.JSON: 'JSON'>, 'CHAR': <TokenType.CHAR: 'CHAR'>, 'CHARACTER': <TokenType.CHAR: 'CHAR'>, 'NCHAR': <TokenType.NCHAR: 'NCHAR'>, 'VARCHAR': <TokenType.VARCHAR: 'VARCHAR'>, 'VARCHAR2': <TokenType.VARCHAR: 'VARCHAR'>, 'NVARCHAR': <TokenType.NVARCHAR: 'NVARCHAR'>, 'NVARCHAR2': <TokenType.NVARCHAR: 'NVARCHAR'>, 'STR': <TokenType.TEXT: 'TEXT'>, 'STRING': <TokenType.TEXT: 'TEXT'>, 'TEXT': <TokenType.TEXT: 'TEXT'>, 'CLOB': <TokenType.TEXT: 'TEXT'>, 'LONGVARCHAR': <TokenType.TEXT: 'TEXT'>, 'BINARY': <TokenType.BINARY: 'BINARY'>, 'BLOB': <TokenType.VARBINARY: 'VARBINARY'>, 'BYTEA': <TokenType.VARBINARY: 'VARBINARY'>, 'VARBINARY': <TokenType.VARBINARY: 'VARBINARY'>, 'TIME': <TokenType.TIME: 'TIME'>, 'TIMESTAMP': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMPTZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPLTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'DATE': <TokenType.DATE: 'DATE'>, 'DATETIME': <TokenType.DATETIME: 'DATETIME'>, 'INT4RANGE': <TokenType.INT4RANGE: 'INT4RANGE'>, 'INT4MULTIRANGE': <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, 'INT8RANGE': <TokenType.INT8RANGE: 'INT8RANGE'>, 'INT8MULTIRANGE': <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, 'NUMRANGE': <TokenType.NUMRANGE: 'NUMRANGE'>, 'NUMMULTIRANGE': <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, 'TSRANGE': <TokenType.TSRANGE: 'TSRANGE'>, 'TSMULTIRANGE': <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, 'TSTZRANGE': <TokenType.TSTZRANGE: 'TSTZRANGE'>, 'TSTZMULTIRANGE': <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, 'DATERANGE': <TokenType.DATERANGE: 'DATERANGE'>, 'DATEMULTIRANGE': <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, 'UNIQUE': <TokenType.UNIQUE: 'UNIQUE'>, 'STRUCT': <TokenType.STRUCT: 'STRUCT'>, 'VARIANT': <TokenType.VARIANT: 'VARIANT'>, 'ALTER': <TokenType.ALTER: 'ALTER'>, 'ANALYZE': <TokenType.COMMAND: 'COMMAND'>, 'CALL': <TokenType.COMMAND: 'COMMAND'>, 'COMMENT': <TokenType.COMMENT: 'COMMENT'>, 'COPY': <TokenType.COMMAND: 'COMMAND'>, 'EXPLAIN': <TokenType.COMMAND: 'COMMAND'>, 'GRANT': <TokenType.COMMAND: 'COMMAND'>, 'OPTIMIZE': <TokenType.COMMAND: 'COMMAND'>, 'PREPARE': <TokenType.COMMAND: 'COMMAND'>, 'TRUNCATE': <TokenType.COMMAND: 'COMMAND'>, 'VACUUM': <TokenType.COMMAND: 'COMMAND'>, 'USER-DEFINED': <TokenType.USERDEFINED: 'USERDEFINED'>, 'BYTEINT': <TokenType.INT: 'INT'>, 'CHAR VARYING': <TokenType.VARCHAR: 'VARCHAR'>, 'CHARACTER VARYING': <TokenType.VARCHAR: 'VARCHAR'>, 'EXCLUDE': <TokenType.EXCEPT: 'EXCEPT'>, 'ILIKE ANY': <TokenType.ILIKE_ANY: 'ILIKE_ANY'>, 'LIKE ANY': <TokenType.LIKE_ANY: 'LIKE_ANY'>, 'MATCH_RECOGNIZE': <TokenType.MATCH_RECOGNIZE: 'MATCH_RECOGNIZE'>, 'MINUS': <TokenType.EXCEPT: 'EXCEPT'>, 'NCHAR VARYING': <TokenType.VARCHAR: 'VARCHAR'>, 'PUT': <TokenType.COMMAND: 'COMMAND'>, 'RENAME': <TokenType.REPLACE: 'REPLACE'>, 'SAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TIMESTAMP_LTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'TIMESTAMP_NTZ': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMP_TZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPNTZ': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TOP': <TokenType.TOP: 'TOP'>}
SINGLE_TOKENS = {'(': <TokenType.L_PAREN: 'L_PAREN'>, ')': <TokenType.R_PAREN: 'R_PAREN'>, '[': <TokenType.L_BRACKET: 'L_BRACKET'>, ']': <TokenType.R_BRACKET: 'R_BRACKET'>, '{': <TokenType.L_BRACE: 'L_BRACE'>, '}': <TokenType.R_BRACE: 'R_BRACE'>, '&': <TokenType.AMP: 'AMP'>, '^': <TokenType.CARET: 'CARET'>, ':': <TokenType.COLON: 'COLON'>, ',': <TokenType.COMMA: 'COMMA'>, '.': <TokenType.DOT: 'DOT'>, '-': <TokenType.DASH: 'DASH'>, '=': <TokenType.EQ: 'EQ'>, '>': <TokenType.GT: 'GT'>, '<': <TokenType.LT: 'LT'>, '%': <TokenType.MOD: 'MOD'>, '!': <TokenType.NOT: 'NOT'>, '|': <TokenType.PIPE: 'PIPE'>, '+': <TokenType.PLUS: 'PLUS'>, ';': <TokenType.SEMICOLON: 'SEMICOLON'>, '/': <TokenType.SLASH: 'SLASH'>, '\\': <TokenType.BACKSLASH: 'BACKSLASH'>, '*': <TokenType.STAR: 'STAR'>, '~': <TokenType.TILDA: 'TILDA'>, '?': <TokenType.PLACEHOLDER: 'PLACEHOLDER'>, '@': <TokenType.PARAMETER: 'PARAMETER'>, "'": <TokenType.QUOTE: 'QUOTE'>, '`': <TokenType.IDENTIFIER: 'IDENTIFIER'>, '"': <TokenType.IDENTIFIER: 'IDENTIFIER'>, '#': <TokenType.HASH: 'HASH'>, '$': <TokenType.PARAMETER: 'PARAMETER'>}
VAR_SINGLE_TOKENS = {'$'}

##### Inherited Members

sqlglot.tokens.Tokenizer
  BIT_STRINGS
  BYTE_STRINGS
  RAW_STRINGS
  IDENTIFIERS
  IDENTIFIER_ESCAPES
  IDENTIFIERS_CAN_START_WITH_DIGIT
  WHITE_SPACE
  COMMANDS
  COMMAND_PREFIX_TOKENS
  NUMERIC_LITERALS
  ENCODE
  reset
  tokenize
  peek
  size
  sql
  tokens

class Snowflake.Generator(sqlglot.generator.Generator): View Source

    324    class Generator(generator.Generator):
    325        PARAMETER_TOKEN = "$"
    326        MATCHED_BY_SOURCE = False
    327        SINGLE_STRING_INTERVAL = True
    328        JOIN_HINTS = False
    329        TABLE_HINTS = False
    330
    331        TRANSFORMS = {
    332            **generator.Generator.TRANSFORMS,
    333            exp.Array: inline_array_sql,
    334            exp.ArrayConcat: rename_func("ARRAY_CAT"),
    335            exp.ArrayJoin: rename_func("ARRAY_TO_STRING"),
    336            exp.AtTimeZone: lambda self, e: self.func(
    337                "CONVERT_TIMEZONE", e.args.get("zone"), e.this
    338            ),
    339            exp.DateAdd: lambda self, e: self.func("DATEADD", e.text("unit"), e.expression, e.this),
    340            exp.DateDiff: lambda self, e: self.func(
    341                "DATEDIFF", e.text("unit"), e.expression, e.this
    342            ),
    343            exp.DateStrToDate: datestrtodate_sql,
    344            exp.DataType: _datatype_sql,
    345            exp.DayOfWeek: rename_func("DAYOFWEEK"),
    346            exp.Extract: rename_func("DATE_PART"),
    347            exp.If: rename_func("IFF"),
    348            exp.LogicalAnd: rename_func("BOOLAND_AGG"),
    349            exp.LogicalOr: rename_func("BOOLOR_AGG"),
    350            exp.Map: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
    351            exp.Max: max_or_greatest,
    352            exp.Min: min_or_least,
    353            exp.PartitionedByProperty: lambda self, e: f"PARTITION BY {self.sql(e, 'this')}",
    354            exp.Select: transforms.preprocess([transforms.eliminate_distinct_on]),
    355            exp.StarMap: rename_func("OBJECT_CONSTRUCT"),
    356            exp.StrPosition: lambda self, e: self.func(
    357                "POSITION", e.args.get("substr"), e.this, e.args.get("position")
    358            ),
    359            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
    360            exp.Struct: lambda self, e: self.func(
    361                "OBJECT_CONSTRUCT",
    362                *(arg for expression in e.expressions for arg in expression.flatten()),
    363            ),
    364            exp.TimeStrToTime: timestrtotime_sql,
    365            exp.TimeToUnix: lambda self, e: f"EXTRACT(epoch_second FROM {self.sql(e, 'this')})",
    366            exp.TimeToStr: lambda self, e: self.func(
    367                "TO_CHAR", exp.cast(e.this, "timestamp"), self.format_time(e)
    368            ),
    369            exp.TimestampTrunc: timestamptrunc_sql,
    370            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    371            exp.Trim: lambda self, e: self.func("TRIM", e.this, e.expression),
    372            exp.TsOrDsToDate: ts_or_ds_to_date_sql("snowflake"),
    373            exp.UnixToTime: _unix_to_time_sql,
    374            exp.VarMap: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
    375        }
    376
    377        TYPE_MAPPING = {
    378            **generator.Generator.TYPE_MAPPING,
    379            exp.DataType.Type.TIMESTAMP: "TIMESTAMPNTZ",
    380        }
    381
    382        STAR_MAPPING = {
    383            "except": "EXCLUDE",
    384            "replace": "RENAME",
    385        }
    386
    387        PROPERTIES_LOCATION = {
    388            **generator.Generator.PROPERTIES_LOCATION,
    389            exp.SetProperty: exp.Properties.Location.UNSUPPORTED,
    390            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    391        }
    392
    393        def except_op(self, expression: exp.Except) -> str:
    394            if not expression.args.get("distinct", False):
    395                self.unsupported("EXCEPT with All is not supported in Snowflake")
    396            return super().except_op(expression)
    397
    398        def intersect_op(self, expression: exp.Intersect) -> str:
    399            if not expression.args.get("distinct", False):
    400                self.unsupported("INTERSECT with All is not supported in Snowflake")
    401            return super().intersect_op(expression)
    402
    403        def describe_sql(self, expression: exp.Describe) -> str:
    404            # Default to table if kind is unknown
    405            kind_value = expression.args.get("kind") or "TABLE"
    406            kind = f" {kind_value}" if kind_value else ""
    407            this = f" {self.sql(expression, 'this')}"
    408            return f"DESCRIBE{kind}{this}"
    409
    410        def generatedasidentitycolumnconstraint_sql(
    411            self, expression: exp.GeneratedAsIdentityColumnConstraint
    412        ) -> str:
    413            start = expression.args.get("start")
    414            start = f" START {start}" if start else ""
    415            increment = expression.args.get("increment")
    416            increment = f" INCREMENT {increment}" if increment else ""
    417            return f"AUTOINCREMENT{start}{increment}"

Generator converts a given syntax tree to the corresponding SQL string.

###### Arguments:

- pretty: Whether or not to format the produced SQL string. Default: False.
- identify: Determines when an identifier should be quoted. Possible values are: False (default): Never quote, except in cases where it's mandatory by the dialect. True or 'always': Always quote. 'safe': Only quote identifiers that are case insensitive.
- normalize: Whether or not to normalize identifiers to lowercase. Default: False.
- pad: Determines the pad size in a formatted string. Default: 2.
- indent: Determines the indentation size in a formatted string. Default: 2.
- normalize_functions: Whether or not to normalize all function names. Possible values are: "upper" or True (default): Convert names to uppercase. "lower": Convert names to lowercase. False: Disables function name normalization.
- unsupported_level: Determines the generator's behavior when it encounters unsupported expressions. Default ErrorLevel.WARN.
- max_unsupported: Maximum number of unsupported messages to include in a raised UnsupportedError. This is only relevant if unsupported_level is ErrorLevel.RAISE. Default: 3
- leading_comma: Determines whether or not the comma is leading or trailing in select expressions. This is only relevant when generating in pretty mode. Default: False
- max_text_width: The max number of characters in a segment before creating new lines in pretty mode. The default is on the smaller end because the length only represents a segment and not the true line length. Default: 80
- comments: Whether or not to preserve comments in the output SQL code. Default: True

PARAMETER_TOKEN = '$'
MATCHED_BY_SOURCE = False
SINGLE_STRING_INTERVAL = True
JOIN_HINTS = False
TABLE_HINTS = False
TRANSFORMS = {<class 'sqlglot.expressions.DateAdd'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.TsOrDsAdd'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CaseSpecificColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CheckColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CollateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CommentColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DateFormatColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DefaultColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EncodeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExternalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InlineLengthColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LanguageProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LocationProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LogProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.MaterializedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnCommitProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnUpdateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.PathColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ReturnsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SettingsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StabilityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TemporaryProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransientProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TitleColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UppercaseColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.VolatileProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.Array'>: <function inline_array_sql>, <class 'sqlglot.expressions.ArrayConcat'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayJoin'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.AtTimeZone'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.DateDiff'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.DateStrToDate'>: <function datestrtodate_sql>, <class 'sqlglot.expressions.DataType'>: <function _datatype_sql>, <class 'sqlglot.expressions.DayOfWeek'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Extract'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.If'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalAnd'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalOr'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Map'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.Max'>: <function max_or_greatest>, <class 'sqlglot.expressions.Min'>: <function min_or_least>, <class 'sqlglot.expressions.PartitionedByProperty'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.Select'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.StarMap'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.StrPosition'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.StrToTime'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.Struct'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.TimeStrToTime'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeToUnix'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToStr'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.TimestampTrunc'>: <function timestamptrunc_sql>, <class 'sqlglot.expressions.ToChar'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.Trim'>: <function Snowflake.Generator.<lambda>>, <class 'sqlglot.expressions.TsOrDsToDate'>: <function ts_or_ds_to_date_sql.<locals>._ts_or_ds_to_date_sql>, <class 'sqlglot.expressions.UnixToTime'>: <function _unix_to_time_sql>}
TYPE_MAPPING = {<Type.NCHAR: 'NCHAR'>: 'CHAR', <Type.NVARCHAR: 'NVARCHAR'>: 'VARCHAR', <Type.MEDIUMTEXT: 'MEDIUMTEXT'>: 'TEXT', <Type.LONGTEXT: 'LONGTEXT'>: 'TEXT', <Type.MEDIUMBLOB: 'MEDIUMBLOB'>: 'BLOB', <Type.LONGBLOB: 'LONGBLOB'>: 'BLOB', <Type.INET: 'INET'>: 'INET', <Type.TIMESTAMP: 'TIMESTAMP'>: 'TIMESTAMPNTZ'}
STAR_MAPPING = {'except': 'EXCLUDE', 'replace': 'RENAME'}
PROPERTIES_LOCATION = {<class 'sqlglot.expressions.AlgorithmProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.AutoIncrementProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BlockCompressionProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CharacterSetProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ChecksumProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CollateProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Cluster'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ClusteredByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DataBlocksizeProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.DefinerProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DictRange'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistStyleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.EngineProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExternalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.FallbackProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.FileFormatProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.FreespaceProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.IsolatedLoadingProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.JournalProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.LanguageProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LikeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LocationProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockingProperty'>: <Location.POST_ALIAS: 'POST_ALIAS'>, <class 'sqlglot.expressions.LogProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.MaterializedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeBlockRatioProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.OnCommitProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.Order'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedByProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.PrimaryKey'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Property'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.ReturnsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatDelimitedProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatSerdeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SerdeProperties'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Set'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SettingsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SetProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.SortKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.StabilityProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TemporaryProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ToTableProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TransientProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeTreeTTL'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.VolatileProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.WithDataProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <Location.POST_NAME: 'POST_NAME'>}
def except_op(self, expression: sqlglot.expressions.Except) -> str: View Source

    393        def except_op(self, expression: exp.Except) -> str:
    394            if not expression.args.get("distinct", False):
    395                self.unsupported("EXCEPT with All is not supported in Snowflake")
    396            return super().except_op(expression)

def intersect_op(self, expression: sqlglot.expressions.Intersect) -> str: View Source

    398        def intersect_op(self, expression: exp.Intersect) -> str:
    399            if not expression.args.get("distinct", False):
    400                self.unsupported("INTERSECT with All is not supported in Snowflake")
    401            return super().intersect_op(expression)

def describe_sql(self, expression: sqlglot.expressions.Describe) -> str: View Source

    403        def describe_sql(self, expression: exp.Describe) -> str:
    404            # Default to table if kind is unknown
    405            kind_value = expression.args.get("kind") or "TABLE"
    406            kind = f" {kind_value}" if kind_value else ""
    407            this = f" {self.sql(expression, 'this')}"
    408            return f"DESCRIBE{kind}{this}"

def generatedasidentitycolumnconstraint_sql( self, expression: sqlglot.expressions.GeneratedAsIdentityColumnConstraint) -> str: View Source

    410        def generatedasidentitycolumnconstraint_sql(
    411            self, expression: exp.GeneratedAsIdentityColumnConstraint
    412        ) -> str:
    413            start = expression.args.get("start")
    414            start = f" START {start}" if start else ""
    415            increment = expression.args.get("increment")
    416            increment = f" INCREMENT {increment}" if increment else ""
    417            return f"AUTOINCREMENT{start}{increment}"

SELECT_KINDS: Tuple[str, ...] = ()
INVERSE_TIME_MAPPING: Dict[str, str] = {'%Y': 'yyyy', '%y': 'yy', '%B': 'mmmm', '%b': 'mon', '%m': 'mm', '%d': 'DD', '%-d': 'dd', '%a': 'DY', '%w': 'dy', '%H': 'hh24', '%I': 'hh12', '%M': 'mi', '%S': 'ss', '%f': 'ff6'}
INVERSE_TIME_TRIE: Dict = {'%': {'Y': {0: True}, 'y': {0: True}, 'B': {0: True}, 'b': {0: True}, 'm': {0: True}, 'd': {0: True}, '-': {'d': {0: True}}, 'a': {0: True}, 'w': {0: True}, 'H': {0: True}, 'I': {0: True}, 'M': {0: True}, 'S': {0: True}, 'f': {0: True}}}
NULL_ORDERING = 'nulls_are_large'
@classmethod
def can_identify(text: str, identify: str | bool = 'safe') -> bool: View Source

    247    @classmethod
    248    def can_identify(cls, text: str, identify: str | bool = "safe") -> bool:
    249        """Checks if text can be identified given an identify option.
    250
    251        Args:
    252            text: The text to check.
    253            identify:
    254                "always" or `True`: Always returns true.
    255                "safe": True if the identifier is case-insensitive.
    256
    257        Returns:
    258            Whether or not the given text can be identified.
    259        """
    260        if identify is True or identify == "always":
    261            return True
    262
    263        if identify == "safe":
    264            return not cls.case_sensitive(text)
    265
    266        return False

Checks if text can be identified given an identify option.

###### Arguments:

- text: The text to check.
- identify: "always" or `True`: Always returns true. "safe": True if the identifier is case-insensitive.

###### Returns:

> Whether or not the given text can be identified.

QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
STRING_ESCAPE = '\\'
IDENTIFIER_ESCAPE = '"'
BIT_START: Optional[str] = None
BIT_END: Optional[str] = None
HEX_START: Optional[str] = "x'"
HEX_END: Optional[str] = "'"
BYTE_START: Optional[str] = None
BYTE_END: Optional[str] = None
RAW_START: Optional[str] = None
RAW_END: Optional[str] = None

##### Inherited Members

sqlglot.generator.Generator
  Generator
  NULL_ORDERING_SUPPORTED
  LOCKING_READS_SUPPORTED
  EXPLICIT_UNION
  WRAP_DERIVED_VALUES
  CREATE_FUNCTION_RETURN_AS
  INTERVAL_ALLOWS_PLURAL_FORM
  TABLESAMPLE_WITH_METHOD
  TABLESAMPLE_SIZE_IS_PERCENT
  LIMIT_FETCH
  RENAME_TABLE_WITH_DB
  GROUPINGS_SEP
  INDEX_ON
  IS_BOOL_ALLOWED
  TIME_PART_SINGULARS
  TOKEN_MAPPING
  STRUCT_DELIMITER
  RESERVED_KEYWORDS
  WITH_SEPARATED_COMMENTS
  UNWRAPPED_INTERVAL_VALUES
  SENTINEL_LINE_BREAK
  INDEX_OFFSET
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  STRICT_STRING_CONCAT
  NORMALIZE_FUNCTIONS
  pretty
  identify
  normalize
  pad
  unsupported_level
  max_unsupported
  leading_comma
  max_text_width
  comments
  normalize_functions
  unsupported_messages
  generate
  unsupported
  sep
  seg
  pad_comment
  maybe_comment
  wrap
  no_identify
  normalize_func
  indent
  sql
  uncache_sql
  cache_sql
  characterset_sql
  column_sql
  columnposition_sql
  columndef_sql
  columnconstraint_sql
  autoincrementcolumnconstraint_sql
  compresscolumnconstraint_sql
  notnullcolumnconstraint_sql
  primarykeycolumnconstraint_sql
  uniquecolumnconstraint_sql
  createable_sql
  create_sql
  clone_sql
  prepend_ctes
  with_sql
  cte_sql
  tablealias_sql
  bitstring_sql
  hexstring_sql
  bytestring_sql
  rawstring_sql
  datatypesize_sql
  datatype_sql
  directory_sql
  delete_sql
  drop_sql
  except_sql
  fetch_sql
  filter_sql
  hint_sql
  index_sql
  identifier_sql
  inputoutputformat_sql
  national_sql
  partition_sql
  properties_sql
  root_properties
  properties
  with_properties
  locate_properties
  property_sql
  likeproperty_sql
  fallbackproperty_sql
  journalproperty_sql
  freespaceproperty_sql
  checksumproperty_sql
  mergeblockratioproperty_sql
  datablocksizeproperty_sql
  blockcompressionproperty_sql
  isolatedloadingproperty_sql
  lockingproperty_sql
  withdataproperty_sql
  insert_sql
  intersect_sql
  introducer_sql
  pseudotype_sql
  onconflict_sql
  returning_sql
  rowformatdelimitedproperty_sql
  withtablehint_sql
  indextablehint_sql
  table_sql
  tablesample_sql
  pivot_sql
  tuple_sql
  update_sql
  values_sql
  var_sql
  into_sql
  from_sql
  group_sql
  having_sql
  join_sql
  lambda_sql
  lateral_sql
  limit_sql
  offset_sql
  setitem_sql
  set_sql
  pragma_sql
  lock_sql
  literal_sql
  escape_str
  loaddata_sql
  null_sql
  boolean_sql
  order_sql
  cluster_sql
  distribute_sql
  sort_sql
  ordered_sql
  matchrecognize_sql
  query_modifiers
  offset_limit_modifiers
  after_having_modifiers
  after_limit_modifiers
  select_sql
  schema_sql
  schema_columns_sql
  star_sql
  parameter_sql
  sessionparameter_sql
  placeholder_sql
  subquery_sql
  qualify_sql
  union_sql
  union_op
  unnest_sql
  where_sql
  window_sql
  partition_by_sql
  windowspec_sql
  withingroup_sql
  between_sql
  bracket_sql
  safebracket_sql
  all_sql
  any_sql
  exists_sql
  case_sql
  constraint_sql
  nextvaluefor_sql
  extract_sql
  trim_sql
  safeconcat_sql
  check_sql
  foreignkey_sql
  primarykey_sql
  if_sql
  matchagainst_sql
  jsonkeyvalue_sql
  jsonobject_sql
  openjsoncolumndef_sql
  openjson_sql
  in_sql
  in_unnest_op
  interval_sql
  return_sql
  reference_sql
  anonymous_sql
  paren_sql
  neg_sql
  not_sql
  alias_sql
  aliases_sql
  attimezone_sql
  add_sql
  and_sql
  connector_sql
  bitwiseand_sql
  bitwiseleftshift_sql
  bitwisenot_sql
  bitwiseor_sql
  bitwiserightshift_sql
  bitwisexor_sql
  cast_sql
  currentdate_sql
  collate_sql
  command_sql
  comment_sql
  mergetreettlaction_sql
  mergetreettl_sql
  transaction_sql
  commit_sql
  rollback_sql
  altercolumn_sql
  renametable_sql
  altertable_sql
  droppartition_sql
  addconstraint_sql
  distinct_sql
  ignorenulls_sql
  respectnulls_sql
  intdiv_sql
  dpipe_sql
  safedpipe_sql
  div_sql
  overlaps_sql
  distance_sql
  dot_sql
  eq_sql
  escape_sql
  glob_sql
  gt_sql
  gte_sql
  ilike_sql
  ilikeany_sql
  is_sql
  like_sql
  likeany_sql
  similarto_sql
  lt_sql
  lte_sql
  mod_sql
  mul_sql
  neq_sql
  nullsafeeq_sql
  nullsafeneq_sql
  or_sql
  slice_sql
  sub_sql
  trycast_sql
  use_sql
  binary
  function_fallback_sql
  func
  format_args
  text_width
  format_time
  expressions
  op_expressions
  naked_property
  set_operation
  tag_sql
  token_sql
  userdefinedfunction_sql
  joinhint_sql
  kwarg_sql
  when_sql
  merge_sql
  tochar_sql
  dictproperty_sql
  dictrange_sql
  dictsubproperty_sql
  oncluster_sql
  clusteredbyproperty_sql
  anyvalue_sql